package org.apcocoa.eclipse;

/**
 * The ApCoCoA Eclipse GUI heavily depends on <i>external contributions</i>
 * such as the CoCoA interpreter, the ApCoCoAServer, the packages directory
 * which is parsed to support code completion etc.
 * 
 * These external contributions are supplied as plug-ins extending the 
 * Extension Point 
 * <code>org.apcocoa.external.contribution</code>
 * 
 * This interface defines the basic methods that should be implemented
 * by an accompanying class such that it can register the necessary paths etc.
 * 
 * Currently, the contribution plug-ins are singleton, so only one contribution 
 * version can be installed.
 * 
 * Access to the installed contribution is provided by 
 * <code>ApCoCoAPlugin</code>.
 * 
 * @author Stefan Schuster
 * @version 1.0
 *
 */
public interface IExternalContribution {

	public static String PROPERTY_COCOA_TEXT_PATH 		= "org.apcocoa.path.cocoa_text";
	
	public static String PROPERTY_APCOCOA_SERVER_PATH 	= "org.apcocoa.path.apcocoa_server";
	
	public static String PROPERTY_PACKAGES_PATH 		= "org.apcocoa.path.packages";
	
	public static String PROPERTY_WORDLIST_PATH 		= "org.apcocoa.path.wordlist";
	
	public static String PROPERTY_COCOARC_PATH 			= "org.apcocoa.path.cocoarc";
	
	/**
	 * Get some information about the installed plug-in.
	 * @return some information about the installed plug-in.
	 * @since 1.0
	 */
	public String getInfo();
	
	/**
	 * Retrieve the absolute path to the folder <code>$os$/apcocoa</code> as
	 * a <code>String</code>.
	 * 
	 * @return the absolute path to the folder <code>$os$/apcooca</code> as a
	 * <code>String</code>.
	 * @since 1.0
	 */
	public String getPluginLocation();
	
	/**
	 * Retrieve the absolute path to the paths defined by this contribution as
	 * a <code>String</code>.
	 * 
	 * @param key one of the <code>IExternalContribution.PROPERTY_*</code> constants.
	 * @return the absolute path to the paths defined by this contribution as
	 * a <code>String</code>.
	 * @since 1.0
	 */
	public String getPathFor(String key);
	
}
