package org.apcocoa.eclipse.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;

import org.apcocoa.eclipse.editors.RecursiveFileTraverser;

public class DocumentationGrepper {

	private Map<String, File> gDocumentMapper = new HashMap<String, File>();

	private File gDocRoot;

	private class DocFileTraverser extends RecursiveFileTraverser {

		public DocFileTraverser(File parentDir) {
			super(parentDir);
		}

		@Override
		protected void handleFile(File file) {
			if (file.getName().startsWith("cmd")
					&& file.getName().endsWith(".html")) {
				String tCommandName = file.getName();
				tCommandName = tCommandName.substring("cmd".length(),
						tCommandName.length() - ".html".length());
				gDocumentMapper.put(tCommandName, file);
			}
		}
	}

	public DocumentationGrepper(String s) throws FileNotFoundException {
		gDocRoot = new File(s);
		if (!gDocRoot.canRead())
			throw new FileNotFoundException();
		initialize();
	}

	private void initialize() {
		// We assume that parent directory contains folders 'cocoa' and
		// 'apcocoa'. These are traversed recursively.
		traverseFolder("cocoa");
		traverseFolder("apcocoa");
	}

	/**
	 * 
	 */
	private void traverseFolder(String subfolderOfDocRoot) {
		String tRelevantSubdirString = gDocRoot.getAbsolutePath() + "/"
				+ subfolderOfDocRoot;
		File tRelevantSubdir = new File(tRelevantSubdirString);
		DocFileTraverser dft = new DocFileTraverser(tRelevantSubdir);
		dft.traverse();
	}

	public File getDocumentationRoot() {
		return gDocRoot;
	}

	
	public String getDocForFunction(String functionName) {
		if (gDocumentMapper.containsKey(functionName))
			return null;
		else
			return null;
	}
	
	private void printMap() {
		for (String s : gDocumentMapper.keySet()) {
			System.out.println(s + " -> " + gDocumentMapper.get(s));
		}
		System.out.println("Map contains " + gDocumentMapper.size()
				+ " entries.");
	}

	public static void main(String[] args) throws FileNotFoundException {
		String tDocRoot = "/Applications/ApCoCoA-1.3.1/plugins/"
				+ "org.apcocoa.contribution.macos.fragment_1.5.1.201011091352/"
				+ "os/macosx/apcocoa/ApCoCoA-1.5.1.app/Contents/MacOS/doc";
		DocumentationGrepper g = new DocumentationGrepper(tDocRoot);
		g.printMap();
	}
}
