package org.apcocoa.eclipse.editor2.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.dltk.ui.text.AbstractScriptScanner;
import org.eclipse.dltk.ui.text.IColorManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWhitespaceDetector;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;

public class ApCoCoACodeScanner extends AbstractScriptScanner {

	private static String[] fgKeywords = { "for", 
		"Define",
		"EndDefine",
		"endfor", 
		"while", 
		"endwhile", 
		"do", 
		"if", 
		"endif",
		"then",
		"else",
		"elsif",
		"elif",
		"alias",
		"break",
		"foreach",
		"endforeach",
		"repeat",
		"endrepeat",
		"print",
		"println",
		"to",
		"or",
		"not",
		"using",
		"endusing" };

	private static String fgTokenProperties[] = new String[] {
			IApCoCoAColorConstants.APCOCOA_COMMENT,
			IApCoCoAColorConstants.APCOCOA_DEFAULT,
			IApCoCoAColorConstants.APCOCOA_KEYWORD };
	
	public ApCoCoACodeScanner(IColorManager manager, IPreferenceStore store) {
		super(manager, store);
		this.initialize();
	}
	
	@Override
	protected String[] getTokenProperties() {
		return fgTokenProperties;
	}

	@Override
	protected List<IRule> createRules() {
		List<IRule> rules = new ArrayList<IRule>();
		
		IToken keyword = this.getToken(IApCoCoAColorConstants.APCOCOA_KEYWORD);
		IToken comment = this.getToken(IApCoCoAColorConstants.APCOCOA_COMMENT);
		IToken other = this.getToken(IApCoCoAColorConstants.APCOCOA_DEFAULT);
		
		// Add rules for single line comments.
		rules.add(new EndOfLineRule("--", comment));
		rules.add(new EndOfLineRule("//", comment));
		// Add generic whitespace rule.
		rules.add(new WhitespaceRule(new ApCoCoAWhitespaceDetector()));
		// Add word rule for keywords.
		WordRule wordRule = new WordRule(new ApCoCoAWordDetector(), other, true);
		
		for (int i = 0; i < fgKeywords.length; i++) {
			wordRule.addWord(fgKeywords[i], keyword);
		}
		rules.add(wordRule);
		this.setDefaultReturnToken(other);
		return rules;
	}
	
	public class ApCoCoAWhitespaceDetector implements IWhitespaceDetector {
		public boolean isWhitespace(char character) {
			return Character.isWhitespace(character);
		}
	}
	public class ApCoCoAWordDetector implements IWordDetector {
		public boolean isWordPart(char character) {
			return Character.isJavaIdentifierPart(character);
		}
		public boolean isWordStart(char character) {
			return Character.isUpperCase(character);
		}
	}
}
