package org.apcocoa.eclipse.internal.interpreter;

import java.io.IOException;

import org.apcocoa.eclipse.ApCoCoANature;
import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IExternalContribution;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.dltk.core.environment.EnvironmentPathUtils;
import org.eclipse.dltk.core.environment.IDeployment;
import org.eclipse.dltk.core.environment.IEnvironment;
import org.eclipse.dltk.core.environment.IFileHandle;
import org.eclipse.dltk.internal.launching.AbstractInterpreterInstallType;
import org.eclipse.dltk.launching.EnvironmentVariable;
import org.eclipse.dltk.launching.IInterpreterInstall;
import org.eclipse.dltk.launching.LaunchingMessages;
import org.eclipse.dltk.launching.LibraryLocation;
import org.eclipse.osgi.util.NLS;
import org.osgi.framework.Bundle;

public class ApCoCoAInterpreterInstallType extends
		AbstractInterpreterInstallType {

	private static final String[] interpreterNames = {
		
		"cocoa_text", 
		"apcocoa_text", 
		"ApCoCoAText",
		"cocoa",
		"apcocoa"};
	
	// @Override
	public String getName() {
		return "ApCoCoA";
	}

	// @Override
	public String getNatureId() {
		return ApCoCoANature.APCOCOA_NATURE;
	}

	@Override
	protected IInterpreterInstall doCreateInterpreterInstall(String id) {
		return new ApCoCoAInterpreterInstall(this, id);
	}

	@Override
	protected String[] getPossibleInterpreterNames() {
		return interpreterNames;
	}

	@Override
	protected String getPluginId() {
		return ApCoCoAPlugin.PLUGIN_ID;
	}

	@Override
	protected ILog getLog() {
		return ApCoCoAPlugin.getDefault().getLog();
	}

	@Override
	protected IPath createPathFile(IDeployment deployment) throws IOException {
		Bundle bundle = ApCoCoAPlugin.getDefault().getBundle();
		return deployment.add(bundle, "");
	}

	public synchronized LibraryLocation[] getDefaultLibraryLocations(
			final IFileHandle installLocation, EnvironmentVariable[] variables,
			IProgressMonitor monitor) {
		if (monitor != null) {
			monitor.beginTask(
							NLS.bind(LaunchingMessages.AbstractInterpreterInstallType_resolvingLibraryPaths,
										this.getName()), 100);
		}
		
		// First see if there is a 'package' directory in the given
		// installLocation.
		IFileHandle parent = installLocation.getParent();
		IFileHandle packages = parent.getChild("packages");
		String packagePathString;
		if ( packages.exists() ) {
			packagePathString = packages.toOSString();
		} else {
			packagePathString = ApCoCoAPlugin.getDefault().getExternalContribution().getPathFor(IExternalContribution.PROPERTY_PACKAGES_PATH);
		}
		
		IEnvironment env = installLocation.getEnvironment();
		IPath p = new Path(packagePathString);
		LibraryLocation l = new LibraryLocation(EnvironmentPathUtils.getFullPath(env, p)); 
		LibraryLocation[] ls = { l };
		
		if (monitor != null)
			monitor.done();
		
		return ls;
	}
}
