package org.apcocoa.eclipse.internal.interpreter;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.apcocoa.eclipse.IExternalContribution;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.dltk.core.environment.IEnvironment;
import org.eclipse.dltk.core.environment.IExecutionEnvironment;
import org.eclipse.dltk.core.environment.IFileHandle;
import org.eclipse.dltk.launching.InterpreterSearcher;
import org.eclipse.dltk.launching.Messages;

public class ApCoCoAInterpreterSearcher extends InterpreterSearcher {


	protected void searchFast(IProgressMonitor monitor,
			IEnvironment environment, int depth) {
		if (monitor.isCanceled()) {
			return;
		}

		// Path variable
		IExecutionEnvironment exeEnv = (IExecutionEnvironment) environment
				.getAdapter(IExecutionEnvironment.class);
		if (exeEnv == null)
			return;

		monitor.subTask(Messages.InterpreterSearcher_0);
		Map<String, String> env = exeEnv.getEnvironmentVariables(true);

		if (env == null) {
			return;
		}
		String path = null;
		for (final String name : env.keySet()) {
			if (name.equalsIgnoreCase("path")) { //$NON-NLS-1$
				path = env.get(name);
				break;
			}
		}
		if (path == null) {
			return;
		}

		// Folder list
		final String separator = environment.getPathsSeparator();

		final List<IPath> folders = new ArrayList<IPath>();
		String[] res = path.split(separator);
		for (int i = 0; i < res.length; i++) {
			folders.add(Path.fromOSString(res[i]));
		}

		// ApCoCoA specific contribution root
		IExternalContribution c = ApCoCoAPlugin.getDefault().getExternalContribution();
		String cocoa_path = c.getPathFor(IExternalContribution.PROPERTY_COCOA_TEXT_PATH);
		IPath tmpPath = Path.fromOSString(cocoa_path).removeLastSegments(1);
		folders.add(tmpPath);
		
		
		monitor.beginTask(Messages.InterpreterSearcher_1, folders.size());
		for (final IPath folder : folders) {
			IFileHandle f = environment.getFile(folder);
			if (f.isDirectory()) {
				search(f, monitor, depth);
			}
			monitor.worked(1);
		}
		monitor.done();
	}

	
	public ApCoCoAInterpreterSearcher() {
		super();
	}
}
