package org.apcocoa.eclipse.internal.interpreter;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;

import org.apcocoa.eclipse.ApCoCoANature;
import org.apcocoa.eclipse.ApCoCoAPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.variables.VariablesPlugin;
import org.eclipse.dltk.core.environment.EnvironmentManager;
import org.eclipse.dltk.core.environment.EnvironmentPathUtils;
import org.eclipse.dltk.core.environment.IEnvironment;
import org.eclipse.dltk.core.environment.IFileHandle;
import org.eclipse.dltk.internal.launching.InterpreterDefinitionsContainer;
import org.eclipse.dltk.launching.IInterpreterInstall;
import org.eclipse.dltk.launching.IInterpreterInstallType;
import org.eclipse.dltk.launching.InterpreterStandin;
import org.eclipse.dltk.launching.LibraryLocation;
import org.eclipse.dltk.launching.ScriptRuntime;
import org.eclipse.dltk.launching.ScriptRuntime.DefaultInterpreterEntry;

public class InterpreterInstallUtils {

	
	public static void checkInterpreters() {
		ApCoCoAPlugin.log("InterpreterInstallUtils.checkInterpreters() start", null, IStatus.INFO);
		
		
		InterpreterDefinitionsContainer idc = new InterpreterDefinitionsContainer();
		
		// 1) Get current persisted interpreters into idc.
		String interpreterXMLString = ScriptRuntime.getPreferences().getString(ScriptRuntime.PREF_INTERPRETER_XML);
		if (interpreterXMLString.length() > 0) {
			try {
				ByteArrayInputStream inputStream = new ByteArrayInputStream(
						interpreterXMLString.getBytes());
				InterpreterDefinitionsContainer.parseXMLIntoContainer(
						inputStream, idc);
			} catch (IOException ioe) {
				ApCoCoAPlugin.log("InterpreterInstallUtils.checkInterpreters()", ioe);
			}
		}
		
		// 2) Find contributed interpreter.
		List<IInterpreterInstall> l = idc.getInterpreterList();
		
		IInterpreterInstall contributedInterpreter = null;
		for (IInterpreterInstall iInterpreterInstall : l) {
			if (iInterpreterInstall.getId().equals("org.apcocoa.eclipse.contributedInterpreterInstall")) {
				contributedInterpreter = iInterpreterInstall;
			}
		}
		
		// 3) Change the home directory to the apcocoa distributions default.
		if (contributedInterpreter != null) {
			InterpreterStandin standin = new InterpreterStandin(contributedInterpreter);
			final IEnvironment localEnv = EnvironmentManager.getLocalEnvironment();

			IFileHandle homeFile;
			try {
				homeFile = localEnv.getFile(new Path(substitute("${org.apcocoa.cocoaInstallDir}")));
				standin.setInstallLocation(homeFile);
				// The conversion also fires the necessary property change 
				// events in ScriptRuntime.
				standin.convertToRealInterpreter();
			} catch (CoreException e) {
				ApCoCoAPlugin.log("InterpreterInstallUtils.checkInterpreters()", e);
			}		
		} else {
			ApCoCoAPlugin.log("InterpreterInstallUtils.checkInterpreters(): No default interpreter found.", null, IStatus.WARNING);
		}
	}
	
	/**
	 * 
	 */
	public static String getCoCoAPackagesPath() {
		IInterpreterInstall install = getInstallByNatureID(ApCoCoANature.APCOCOA_NATURE);
		if (install == null)
			return null;
		IInterpreterInstallType type = install.getInterpreterInstallType();
		LibraryLocation[] libs = install.getLibraryLocations();
		if (libs == null) {
			libs = type
					.getDefaultLibraryLocations(install.getInstallLocation());
		}
		if (libs == null || libs.length == 0)
			return null;

		String packagePath = null;
		for (LibraryLocation libraryLocation : libs) {
			IPath currLibPath = libraryLocation.getLibraryPath();

			if (currLibPath.lastSegment().equalsIgnoreCase("packages")) {
				IPath p = EnvironmentPathUtils.getLocalPath(currLibPath);
				packagePath = p.toOSString();
			}
		}
		if (packagePath == null) {
			IPath p = EnvironmentPathUtils.getLocalPath(libs[0].getLibraryPath());
			packagePath = p.toOSString();
		}
		return packagePath;
	}

	public static String getCoCoAExecutable() {
		// old:
		// ApCoCoAPlugin.getPlugin().getPreferenceStore().getString(IApCoCoAConstants.PREF_AC_EXEC);
		IInterpreterInstall install = getInstallByNatureID(ApCoCoANature.APCOCOA_NATURE);
		if (install == null)
			return null;
		return install.getInstallLocation().getPath().toString();
	}

	public static IInterpreterInstall getInstallByNatureID(String nature) {
		DefaultInterpreterEntry[] entries = ScriptRuntime
				.getDefaultInterpreterIDs();

		for (DefaultInterpreterEntry entry : entries) {
			String currNature = entry.getNature();
			if (currNature.equals(nature)) {
				IInterpreterInstall install = ScriptRuntime
						.getDefaultInterpreterInstall(entry);
				return install;

			}
		}
		return null;
	}
	
	/**
	 * Performs string substitution on the given expression.
	 * 
	 * @param expression
	 * @return expression after string substitution
	 * @throws CoreException
	 * 
	 */
	private static String substitute(String expression) throws CoreException {
		return VariablesPlugin.getDefault().getStringVariableManager()
				.performStringSubstitution(expression);
	}
}
