package org.apcocoa.launch;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apcocoa.eclipse.internal.interpreter.InterpreterInstallUtils;
import org.eclipse.cdt.utils.spawner.ProcessFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.LaunchConfigurationDelegate;

public class ApcocoaLaunchDelegate extends LaunchConfigurationDelegate {

	// disabled: private ProcessTerminationCheck fTerminationCheck;

	public final static String ATTR_COCOA_PORT = "cocoa_port";

	public final static String ATTR_COCOA_RC = "cocoa_rc";

	public final static String ATTR_COCOA_START_MODE = "cocoa_start_mode";

	public final static String ATTR_COCOA_PACKAGES_PATH = "cocoa_packages_path";

	public ApcocoaLaunchDelegate() {
		super();
	}

	public void launch(ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {

		// TODO We currently ignore the launch and set everything here.

		Process spawner;

		// Set the cocoa_text executable.
		String cocoaPath = InterpreterInstallUtils.getCoCoAExecutable();

		final String errorMessage = "The given ApCoCoA interpreter does not " +
				"or no longer exist." +
				"The corresponding preference page will be opened now and you can " +
				"search for interpreters.";
		
		if (cocoaPath == null) {
			System.out.println("cocoa path is null.");
			throw new CoreException(new Status(IStatus.ERROR,
					"ApcocoaLaunchDelegate",
					errorMessage));
		}
		// Set the working directory.
		File cocoaPathAsFile = new File(cocoaPath);
		if (!cocoaPathAsFile.exists()) {
			System.out.println("cocoa path does not exist.");
			throw new CoreException(new Status(IStatus.ERROR,
					"ApcocoaLaunchDelegate",
					errorMessage));
		}
		File workingDir = (new File(cocoaPath)).getParentFile()
				.getAbsoluteFile();

		
		// If we add new variables to the environment, we must ensure to
		// extend the current environment. This is especially important on
		// win32 since otherwise cocoa_text cannot open sockets to communicate
		// to the server.
		EnvironmentHandler eh = EnvironmentHandler.getInstance();

		
		// Add environment variable COCOA_ROOT
		eh.add("COCOA_ROOT", workingDir.getAbsolutePath());
		
		// Add environment variable COCOA_PACKAGES
		// String tCocoaPackages =
		// configuration.getAttribute(ATTR_COCOA_PACKAGES_PATH, "");
		// if (tCocoaPackages != null && !tCocoaPackages.trim().isEmpty())
		// eh.add("COCOA_PACKAGES", tCocoaPackages);
		// else {
		String tPackages = InterpreterInstallUtils.getCoCoAPackagesPath();
		if (tPackages != null && tPackages.length() != 0)
			eh.add("COCOA_PACKAGES", tPackages);
		
		// }

		// Add environment variable COCOARC
		String tCococaRc = configuration.getAttribute(ATTR_COCOA_RC, "");
		if (tCococaRc != null && !tCococaRc.trim().equals(""))
			eh.add("COCOARC", tCococaRc);

		String[] environment = eh.getEnvironment();

		try {
			ProcessFactory factory = ProcessFactory.getFactory();

			// environment = null : inherit the environment from the current
			// process.
			spawner = factory.exec(cocoaPath, environment, workingDir);

			// -----------------------------------------------------------------
			// This code was only used to experiment with different start
			// options
			// To enable it, you should also un-comment the corresponding code
			// in the
			// preference store.
			// String startMode =
			// configuration.getAttribute(ATTR_COCOA_START_MODE, "SPAWNER");
			// if (startMode.equals(IApCoCoAConstants.COCOA_START_WITH_PTY)) {
			// if (PTY.isSupported()) {
			// PTY pty = new PTY(false);
			// System.out.println("Starting process with pty.");
			// spawner = factory.exec({cocoaPath}, environment, workingDir,
			// pty);
			// } else {
			// System.out.println("Starting process without pty.");
			// spawner = factory.exec(cocoaPath, environment, workingDir);
			// }
			// } else if
			// (startMode.equals(IApCoCoAConstants.COCOA_START_WITHOUT_PTY)) {
			// System.out.println("Starting process without pty.");
			// spawner = factory.exec(cocoaPath, environment, workingDir);
			// } else {
			// System.out.println("Starting process using Runtime.exec()");
			// spawner = Runtime.getRuntime().exec(cocoaPath, environment,
			// workingDir);
			// }
			// -----------------------------------------------------------------

		} catch (IOException exception) {

			Status error;
			String message = exception.getMessage();
			error = new Status(IStatus.ERROR, "TODO ID", message, exception);
			throw new CoreException(error);
		}

		// Currently disabled.
		// fTerminationCheck = new ProcessTerminationCheck(spawner);
		// fTerminationCheck.start();

		Map processAttributes = new HashMap();
		String programName = "apcocoa_text";
		processAttributes.put(IProcess.ATTR_PROCESS_TYPE,
				programName.toLowerCase());

		// Create the IProcess:
		//
		IProcess process = null;
		if (spawner != null) {

			String[] configurationName = { configuration.getName() };
			process = DebugPlugin.newProcess(launch, spawner, cocoaPath,
					processAttributes);
		}
	}

	/**
	 * (Currently Unused) Start a new thread that regularly checks if the cocoa
	 * process has terminated.
	 * 
	 * @author Stefan Schuster
	 * 
	 */
	public class ProcessTerminationCheck extends Thread {

		private Process fProcess;

		private boolean fHasTerminated;

		public ProcessTerminationCheck(Process p) {
			fProcess = p;
		}

		public void run() {
			while (!fHasTerminated) {
				try {
					synchronized (this) {
						this.wait(1000);
					}
					int s = fProcess.exitValue();
					System.out.println("Process is terminated.");
					fHasTerminated = true;
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalThreadStateException e) {
					System.out.println("Process is still running.");
				}
			}
		}
	}

}
