package org.apcocoa.launch;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * 
 * Singleton class to handle the environment variables cocoa is called with.
 * 
 * @author Stefan Schuster
 *
 */
public class EnvironmentHandler {
	
	private static EnvironmentHandler instance = null;
	
	private Map<String, String> gEnv;
	
	private EnvironmentHandler() {
		
	}
	
	public static EnvironmentHandler getInstance() {
		if (instance == null)
			instance = new EnvironmentHandler();
		instance.initialize();
		return instance;
	}
	
	public void initialize() {
		gEnv = new HashMap<String, String>(System.getenv());
	}
	
	public void add(String key, String value) {
		if (key == null || value == null)
			throw new IllegalArgumentException("Parameters 'key' (" + key +
					") and 'value' (" + value + ") must not be 'null'.");
		gEnv.put(key, value);
	}
	
	public String[] getEnvironment() {
		String[] env = new String[gEnv.size()];
		Set<String> keys = gEnv.keySet();
		int i=0;
		for (String k : keys) {
			StringBuilder tBuilder = new StringBuilder();
			tBuilder.append(k);
			tBuilder.append("=");
			tBuilder.append(gEnv.get(k));
			env[i++] = tBuilder.toString();
		}
		return env;
	}
	
	private void printMap()	{
		for (String s : getEnvironment())
			System.out.println(s);
	}
	
	public static void main(String[] args) {
		EnvironmentHandler eh = EnvironmentHandler.getInstance();
		eh.printMap();
		eh.add("APCOCOA_RC", "c:/blah");
		System.out.println("++++++");
		eh.printMap();
	}

}
